# -*- coding: utf-8 -*-
from tg import lurl, config
from tw2.core import CSSLink, JSLink, JSSource
from tw2.forms import TextArea


class MarkupLanguageCSSLink(CSSLink):
    def prepare(self):
        super(MarkupLanguageCSSLink, self).prepare()
        self.link = str(self.link).format(language=config['_flatpages']['format'])


class MarkupLanguageJSLink(JSLink):
    def prepare(self):
        super(MarkupLanguageJSLink, self).prepare()
        self.link = str(self.link).format(language=config['_flatpages']['format'])


class MarkitUpArea(TextArea):
    resources = [
        CSSLink(link=lurl('/_pluggable/flatpages/markitup/skins/simple/style.css')),
        MarkupLanguageCSSLink(link=lurl('/_pluggable/flatpages/markitup/sets/{language}/style.css')),
        JSLink(link=lurl('/_pluggable/flatpages/jquery.js'), location='bodybottom'),
        JSLink(link=lurl('/_pluggable/flatpages/markitup/jquery.markitup.js'), location='bodybottom'),
        MarkupLanguageJSLink(link=lurl('/_pluggable/flatpages/markitup/sets/{language}/set.js'), location='bodybottom'),
        JSSource(src='var jQflatpages = jQuery.noConflict(true);'),
        JSSource(src='jQflatpages("#sx_content").markItUp(mySettings)')
    ]



import docutils
import docutils.core

def rst_formatter(text):
    return docutils.core.publish_parts(text, writer_name='html')['html_body']


def html_formatter(text):
    return text


FORMATTERS = {'rst': rst_formatter,
              'html': html_formatter}


# -*- coding: utf-8 -*-
"""Setup the flatpages application"""
from __future__ import print_function

from flatpages import model
from tgext.pluggable import app_model

def bootstrap(command, conf, vars):
    print('Bootstrapping flatpages...')

    p = app_model.Permission(permission_name='flatpages', description='Can manage flat pages')
    model.DBSession.add(p)
    model.DBSession.flush()

    g = model.DBSession.query(app_model.Group).filter_by(group_name='managers').first()
    if g:
        g.permissions.append(p)
    model.DBSession.flush()




from __future__ import unicode_literals
from contextlib import closing
import os

from sqlalchemy import ForeignKey, Column
from sqlalchemy.types import Unicode, Integer, DateTime
from sqlalchemy.orm import backref, relation

from flatpages.model import DeclarativeBase, DBSession
from tgext.pluggable import app_model, primary_key, plug_url

from datetime import datetime
from tg import config
from tg.caching import cached_property
from flatpages.lib.formatters import FORMATTERS
from depot.fields.sqlalchemy import UploadedFileField


class FlatPage(DeclarativeBase):
    __tablename__ = 'flatpages_page'

    uid = Column(Integer, autoincrement=True, primary_key=True)

    template = Column(Unicode(1024), nullable=False, default="genshi:flatpages.templates.page")
    slug = Column(Unicode(128), index=True, unique=True, nullable=False)
    title = Column(Unicode(512), nullable=False)
    content = Column(Unicode(64000), default='')
    required_permission = Column(Unicode(256), nullable=True, default=None)

    updated_at = Column(DateTime, nullable=False,
                        default=datetime.utcnow, onupdate=datetime.utcnow)
    created_at = Column(DateTime, nullable=False,
                        default=datetime.utcnow)

    author_id = Column(Integer, ForeignKey(primary_key(app_model.User)))
    author = relation(app_model.User)

    @classmethod
    def by_slug(cls, slug):
        return DBSession.query(cls).filter_by(slug=slug).first()

    @cached_property
    def url(self):
        return plug_url('flatpages', '/' + self.slug)

    @cached_property
    def html_content(self):
        format = config['_flatpages']['format']
        formatter = FORMATTERS[format]

        content = self.content
        if content.startswith('file://'):
            package_path = config['paths']['root']
            file_path = os.path.join(package_path, content[7:])
            with closing(open(file_path)) as f:
                content = f.read()

        return formatter(content)


class FlatFile(DeclarativeBase):
    __tablename__ = 'flatpages_file'

    uid = Column(Integer, autoincrement=True, primary_key=True)

    name = Column(Unicode(99), index=True, unique=True, nullable=False)
    file = Column(UploadedFileField(upload_storage='flatfiles'), nullable=False)

    updated_at = Column(DateTime, nullable=False,
                        default=datetime.utcnow, onupdate=datetime.utcnow)
    created_at = Column(DateTime, nullable=False,
                        default=datetime.utcnow)

    author_id = Column(Integer, ForeignKey(primary_key(app_model.User)))
    author = relation(app_model.User)

    @cached_property
    def url(self):
        return plug_url('flatpages', '/flatfiles/' + self.file.file_id)



# -*- coding: utf-8 -*-
"""Main Controller"""
from io import BytesIO
from markupsafe import Markup
from tg import TGController, predicates, config, abort, override_template, tmpl_context, Request
from tg import expose, flash, require, url, lurl, request, redirect, validate
from tg.caching import cached_property
from tg.decorators import with_trailing_slash
from tgext.admin import AdminController, AdminConfig, CrudRestControllerConfig
from tgext.admin.widgets import BootstrapAdminTableFiller
from tgext.crud import EasyCrudRestController, addopts
from tgext.pluggable import primary_key, app_model, plug_url
from tw2.core import Deferred, CSSSource
from tw2.forms import SingleSelectField, TextField, FileValidator

from flatpages import model
from flatpages.lib.widgets import MarkitUpArea
from flatpages.model import DBSession

from tgext.admin.layouts import BootstrapAdminLayout
from depot.middleware import DepotMiddleware
from tg.exceptions import HTTPNotFound
from rst2pdf.createpdf import RstToPdf


class FlatPagesAdminConfig(AdminConfig):
    include_left_menu = False
    layout = BootstrapAdminLayout
    default_index_template = 'genshi:flatpages.templates.manage'

    class flatfile(CrudRestControllerConfig):
        class defaultCrudRestController(EasyCrudRestController):
            response_type = 'text/html'
            remember_values = ['file']

            _get_current_user = lambda: getattr(request.identity['user'], primary_key(app_model.User).key)

            __form_options__ = {
                '__hide_fields__': ['author'],
                '__omit_fields__': ['uid', '_id', 'updated_at', 'created_at'],
                '__field_widget_args__': addopts(author={'value': Deferred(_get_current_user)})
            }

            __form_edit_options__ = {
                '__field_validators__': {'file': FileValidator(required=False)}
            }

            __table_options__ = {
                '__omit_fields__': ['_id', 'uid', 'author_id', 'created_at'],
                '__xml_fields__': ['file'],

                'file': lambda filler, o: Markup('<a href="%s">%s</a>' % (o.url, o.url))
            }

    class flatpage(CrudRestControllerConfig):
        class defaultCrudRestController(EasyCrudRestController):
            response_type = 'text/html'
            crud_resources = [CSSSource(location='headbottom',
                                       src='''
        .crud-sidebar .active {
            font-weight: bold;
            border-left: 3px solid #eee;
        }

        @media (max-width: 991px) {
            .pull-sm-right {
                float: right;
            }
        }

        @media (min-width: 992px) {
            .pull-md-right {
                float: right;
            }
        }
        ''')]

            # Helpers to retrieve form data
            _get_current_user = lambda: getattr(request.identity['user'], primary_key(app_model.User).key)
            _get_templates = lambda: config['_flatpages']['templates']
            _get_permissions = lambda: [('public', 'Public'), ('not_anonymous', 'Only Registered Users')] + \
                                       DBSession.query(app_model.Permission.permission_name,
                                                       app_model.Permission.description).all()

            __form_options__ = {
                '__hide_fields__': ['author'],
                '__omit_fields__': ['uid', '_id', 'updated_at', 'created_at'],
                '__field_order__': ['slug', 'title', 'template', 'required_permission'],
                '__field_widget_types__': addopts(**{'template': SingleSelectField,
                                           'slug': TextField,
                                           'title': TextField,
                                           'required_permission': SingleSelectField,
                                           'content': MarkitUpArea}),
                '__field_widget_args__': addopts(**{'author': {'value': Deferred(_get_current_user)},
                                          'required_permission': {'prompt_text': None,
                                                                  'options': Deferred(_get_permissions)},
                                          'content': {'rows': 20},
                                          'template': {'prompt_text': None,
                                                       'options': Deferred(_get_templates)}})
            }

            __table_options__ = {
                '__omit_fields__': ['_id', 'uid', 'author_id', 'created_at', 'template', 'content'],
                '__xml_fields__': ['slug'],

                'slug': lambda filler, o: Markup('<a href="%s">%s</a>' % (o.url, o.slug)),
                '__actions__': lambda self, obj: request.controller_state.controller._build_actions(obj)
            }

            def _build_actions(self, obj):
                baseactions = super(self.table_filler.__class__, self.table_filler).__actions__(obj)
                if config['_flatpages']['format'] == 'rst':
                    extraactions = Markup('''
        <a href="%s/download" class="btn btn-default">
            <span class="glyphicon glyphicon-download-alt"></span>
        </a>''' % obj.uid)
                else:
                    extraactions = ''
                return extraactions + baseactions

            @expose(content_type='application/pdf')
            def download(self, pageid):
                p = DBSession.query(model.FlatPage).get(pageid) or abort(404)
                out = BytesIO()

                rst2pdf = RstToPdf()
                rst2pdf.createPdf(p.content, output=out)

                out.seek(0)
                return out.read()


class FlatPagesAdminController(AdminController):
    allow_only = predicates.has_permission('flatpages')

    def __init__(self):
        super(FlatPagesAdminController, self).__init__(
            [model.FlatPage,
             model.FlatFile],
            DBSession.wrapped_session,
            config_type=FlatPagesAdminConfig
        )

    @expose()
    @with_trailing_slash
    def index(self, *args, **kwargs):
        return super(FlatPagesAdminController, self).index(*args, **kwargs)


class RootController(TGController):
    CACHE_EXPIRE = 7*86400  # 7 Days

    depotmiddleware = DepotMiddleware(HTTPNotFound(), '/pages')

    @expose()
    def _default(self, page=None, *args, **kw):
        page = model.FlatPage.by_slug(page)
        if page is None:
            abort(404, 'Page not found')

        permission = page.required_permission
        if permission and permission != 'public':
            if permission == 'not_anonymous':
                predicate = predicates.not_anonymous()
            else:
                predicate = predicates.has_permission(permission)

            if not predicate.is_met(request.environ):
                abort(403, 'Forbidden')

        try:
            userid = request.identity['user'].user_id
        except:
            userid = None

        override_template(RootController._default, page.template)
        return dict(page=page,
                    tg_cache={'expire': self.CACHE_EXPIRE,
                              'key': '%s-%s-%s' % (page.slug, page.updated_at, userid)})

    @expose(content_type='text/html')
    def flatfiles(self, *args, **kwargs):
        r = Request.blank('/pages/flatfiles/' + '/'.join(args))
        return r.get_response(self.depotmiddleware)

    @cached_property
    def manage(self):
        return FlatPagesAdminController()



# -*- coding: utf-8 -*-
"""The tgapp-flatpages package"""
import tg


def plugme(app_config, options):
    config = {'templates': options.get('templates', [('genshi:flatpages.templates.page', 'default')]),
              'format': options.get('format', 'rst')}
    app_config['_flatpages'] = config
    return dict(appid='pages', global_helpers=False)


# -*- coding: utf-8 -*-
import sys, os

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

install_requires=[
    "TurboGears2 >= 2.3.0",
    "tgext.pluggable",
    "docutils",
    "tgext.admin >= 0.7.1",
    "tgext.crud >= 0.8.1",
    "genshi",
    "filedepot",
    "rst2pdf"
]

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''

setup(
    name='tgapp-flatpages',
    version='0.2.2',
    description='Pluggable application to manage static pages (/about) on DB with RST',
    long_description=README,
    author='Alessandro Molina',
    author_email='alessandro.molina@axant.it',
    url='https://github.com/amol-/tgapp-flatpages',
    keywords='turbogears2.application',
    packages=find_packages(exclude=['ez_setup']),
    install_requires=install_requires,
    include_package_data=True,
    package_data={'tgapp.flatpages': ['i18n/*/LC_MESSAGES/*.mo',
                                 'templates/*/*',
                                 'public/*/*']},
    entry_points="""
    """,
    zip_safe=False
)
